#include "combinels.h"
#include <cassert>

using namespace realea;

class CombineParams : public ILSParameters {
public:
    int m_type;
    ILSParameters *m_option;

    ~CombineParams(void) { 
	if (m_option != NULL) {
	  delete m_option;
	}
    }
};

CombinedLS::CombinedLS(void) : m_pool() {
   m_initpool = false;
}

void CombinedLS::addLocalSearch(ILocalSearch *ls) {
    m_pool.insert(m_pool.begin(), ls);

}

CombinedLS::~CombinedLS(void) {
    vector<ILocalSearch*>::iterator item;

    for (item = m_pool.begin(); item != m_pool.end(); item++) {
	delete (*item);
	*item = NULL;
    }

}

void CombinedLS::setLSPool(void) {
      
     vector<ILocalSearch*>::iterator item;

    for (item = m_pool.begin(); item != m_pool.end(); item++) {
	(*item)->setPopulation(m_pop);
	(*item)->setRunning(m_running);
	(*item)->setProblem(m_problem);
	(*item)->setRandom(m_random);
	(*item)->setEval(m_eval);
    }
}

ILSParameters *CombinedLS::getInitOptions(tChromosomeReal &sol) {
    int sizeLs = m_pool.size();

    assert(sizeLs > 0);

    if (!m_initpool) {
	setLSPool();
	m_initpool = true;
    }

    int typeLs = m_random->randint(0, sizeLs-1);
    ILocalSearch *ls = m_pool[typeLs];
    CombineParams *options = new CombineParams();
    options->m_type = typeLs;
    options->m_option = ls->getInitOptions(sol);
    return options;
}

unsigned CombinedLS::apply(ILSParameters *opt, tChromosomeReal &sol, tFitness &fitness, unsigned itera) {
    CombineParams *p = (CombineParams *) opt;
    ILocalSearch *ls = m_pool[p->m_type];

    return ls->apply(p->m_option, sol, fitness, itera);
}
